Package: shinyauthr
Type: Package
Title: 'Shiny' Authentication Modules
Version: 1.0.0
Authors@R: 
    c(person(given = "Paul", 
             family = "Campbell", 
             email = "pacampbell91@gmail.com",
             role = c("aut", "cre"),
             comment = c(ORCID = "0000-0003-1018-6606")),
      person(given = "Michael", 
             family = "Dewar", 
             email = "michael.p.dewar@gmail.com", 
             role = "ctb"))
Description: Add in-app user authentication to 'shiny', 
    allowing you to secure publicly hosted apps and 
    build dynamic user interfaces from user information.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: shiny (>= 1.5.0), shinyjs, dplyr, rlang, sodium, glue
Suggests: DBI, RSQLite, lubridate, shinydashboard, testthat (>= 3.0.0),
        shinytest, knitr, rmarkdown, covr
RoxygenNote: 7.1.1
URL: https://github.com/paulc91/shinyauthr
BugReports: https://github.com/paulc91/shinyauthr/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2021-07-19 10:41:29 UTC; paul
Author: Paul Campbell [aut, cre] (<https://orcid.org/0000-0003-1018-6606>),
  Michael Dewar [ctb]
Maintainer: Paul Campbell <pacampbell91@gmail.com>
Repository: CRAN
Date/Publication: 2021-07-20 07:20:02 UTC
Built: R 4.6.0; ; 2026-04-26 11:54:23 UTC; unix
